## MmmmTools   -  Usability Improvements For Maya
## Copyright (C) <2008>  Joseph Crawford
##
## This file is part of MmmmTools.
##
## MmmmTools is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## MmmmTools is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.
##
################################################
## More information is available:
## MmmmTools website - http://celestinestudios.com/mmmmtools
##
## Center Pivot On Component Selection Originally Written by Kert Saville
##
################################################
import math, sys, os
import maya.cmds as cmds
import maya.OpenMaya as om
from pymel.all import *
import pymel.all as pm
import maya.mel



try:
    reload( ModelerSelector )
except:
    pass
import ModelerSelector

try:
    reload( ModelerMirrorer )
except:
    pass
import ModelerMirrorer

try:
    reload( ModelerRetoper )
except:
    pass
import ModelerRetoper

try:
    reload( ModelerAligner )
except:
    pass
import ModelerAligner

try:
    reload( ModelerGridTools )
except:
    pass
import ModelerGridTools

class Modeler(object):
    def __init__(self, parent):
        self.parent = parent
        
    def creaseSelectedEdges( self ):
        objs = ls(selection=True, flatten=True)
        for obj in objs:
            try:
                creaseValue = polyCrease( obj, value = 20.0 )
            except:
                pass
                
    def uncreaseSelectedEdges( self ):
        objs = ls(selection=True, flatten=True)
        for obj in objs:
            try:
                creaseValue = polyCrease( obj, value = 0.0 )
            except:
                pass
                
    def selectCreasedEdges( self ):
        maya.mel.eval( "ConvertSelectionToEdges;" )
        objs = ls(selection=True, flatten=True)
        objsToSelect = []
        for obj in objs:
            try:
                creaseValue = polyCrease( obj, query=True, value=True )
                creaseValue = creaseValue[0]
            except:
                creaseValue = 0
            #print( str(obj) + " is " + str(creaseValue) )        
            if creaseValue > 0.01:
                #print( "found a creased edge" )
                objsToSelect.append(obj)

        if len(objsToSelect) > 0:
            #print(objsToSelect)
            #print("selecting")    
            select( objsToSelect, replace=True )
        else:
            select( clear=True )
            

    def selectHardEdges( self ):
        print( "Selecting Hard Edges" )
        pm.mel.eval( "ConvertSelectionToEdges;" )
        cmps = pm.ls(selection=True, flatten=True )
        to_select = []
        for cmp in cmps:
            if cmp.isSmooth()==False:
                to_select.append( cmp )
        pm.select( to_select )
        
        # pass
        # polySelectConstraint( m=2, t=0x8000, smoothness=1 ) # smoothness: 1 is hard, 2 is smooth, 0 means off
        # polySelectConstraint( m=0 ) # Turn off the constraint        
        #pass
        
        
    def activateSplitPolygonTool( self ):
        pm.mel.eval("SplitPolygonTool;")
          
    def propagateEdgeHardnessOn( self ):
        objs = ls(selection=True, flatten=True)
        objsToSelect = []
        for obj in objs:
            try:
                setAttr( obj+".propagateEdgeHardness", 1 )
            except:
                print("Could not affect " + obj )
                
    def propagateEdgeHardnessOff( self ):
        objs = ls(selection=True, flatten=True)
        objsToSelect = []
        for obj in objs:
            try:
                setAttr( obj+".propagateEdgeHardness", 0 )
            except:
                print("Could not affect " + obj )
                
                
    def centerPivotOnComponents(self):
      ##Convert selection to verts
      sF= (cmds.ls (sl=1))
      print (sF)
      sV= (cmds.polyListComponentConversion( (cmds.ls (sl=1, flatten=1)), tv=True))
      print (sV)
      cmds.select (sV)
    
      ##make list of selected verts
      selectedItems = (cmds.ls (flatten=True, selection=True))
      lengthOfList = len(selectedItems)
      print ("***Number of items in list " + str(lengthOfList))
    
      ##average verts
      ##print (selectedItems)
      xTotal=0
      yTotal=0
      zTotal=0
      for i in xrange(len(selectedItems)):
      ##print ("iteration" + str((i)+1))
        print ("Selected Item in iteration " + str((i)+1) + ": " + str(selectedItems[i]))
        print (cmds.pointPosition (selectedItems[i]))
        xTotal = xTotal + (cmds.pointPosition (selectedItems[i])[0])
        yTotal = yTotal + (cmds.pointPosition (selectedItems[i])[1])
        zTotal = zTotal + (cmds.pointPosition (selectedItems[i])[2])
        i=i+1
    
      xNew = (xTotal)/(lengthOfList)
      yNew = (yTotal)/(lengthOfList)
      zNew = (zTotal)/(lengthOfList)
      print ("center of selected items " + str(xNew) + " " + str(yNew) + " " + str(zNew))
      ##move pivot of selected obj to average of selected verts
      parentObj= (cmds.ls (hilite=True))
      ##print (parentObj[0])
      scalePivotNew = (str(parentObj[0]) + ".scalePivot")
      rotatePivotNew = (str(parentObj[0]) + ".rotatePivot")
      ##print (scalePivotNew)
      ##print (rotatePivotNew)
      cmds.move ((xNew), (yNew), (zNew), scalePivotNew, rotatePivotNew)
      cmds.select (parentObj) ##revert to original selection's parent obj
                
    def runSelector( self, makeUi=False, trisOnly=False, ngonsOnly=False, quadsOnly=False ):
        reload( ModelerSelector )
        self.modelerMeshUtilitiesAndAlignerSelector = ModelerSelector.ModelerSelector( makeUi=makeUi, trisOnly=trisOnly, ngonsOnly=ngonsOnly, quadsOnly=quadsOnly )                  
    def runRetoper( self, makeUi=False ):
        reload( ModelerRetoper )
        self.retoper = ModelerRetoper.ModelerRetoper( makeUi=makeUi )  
    def runAligner( self, makeUi=True ):
        reload( ModelerAligner )
        self.modelerAligner = ModelerAligner.ModelerAligner( makeUi=makeUi )
    def runMirrorer( self, makeUi=True ):
        reload( ModelerMirrorer )
        self.modelerMirrorer = ModelerMirrorer.ModelerMirrorer( makeUi=makeUi )
    def runGridTools( self, makeUi=True ):
        reload( ModelerGridTools )
        self.modelerGridTools = ModelerGridTools.ModelerGridTools( parent=self, makeUi=makeUi )
