## MmmmTools   -  Usability Improvements For Maya
## Copyright (C) <2008>  Joseph Crawford
##
## This file is part of MmmmTools.
##
## MmmmTools is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## MmmmTools is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.
##
################################################
## More information is available:
## MmmmTools website - http://celestinestudios.com/mmmmtools
##
## Center Pivot On Component Selection Originally Written by Kert Saville
##
################################################
import math, sys, os
import traceback
import maya.cmds as cmds
import pymel.all as pm

import maya.OpenMaya as OpenMaya

from . import TexturerUvXformsTool


class Texturer(object):
    def __init__(self,parentRef):
        self.parentRef = parentRef
        self.mmmmTools = self.parentRef
        self.conf = self.mmmmTools.configuration.conf
        self.seamsHiddenString="unknown"
        
    def reloadTextures(self):
        print("Reloading textures for selected objects.")
        try:
            self.mmmmTools.fileTextureReloader.reload()
        except:
            traceback.print_exc()    
        pass
    
    def selectTextureBorderEdges(self):
        print("Selecting seams (texture border edges).")
        try:
            self.mmmmTools.selectTextureBorderEdges.selectTextureBorderEdges()
        except:
            traceback.print_exc()

    
    def selectSeams(self):
        self.selectTextureBorderEdges()
    
    def toggleSeams(self):
        print( "Toggling visibility of seams.")
        if self.seamsHiddenString!="showing":
            self.showSeams()
            self.seamsHiddenString = "showing"
        else:
            self.hideSeams()        
            self.seamsHiddenString = "hidden"
    
    def showSeams(self):
        print( 'Showing seams.'  )
        pm.polyOptions( displayMapBorder=True, gl=True ) ## gl for global
        pm.polyOptions( sizeBorder=self.conf.texturer_show_seams_border_thickness,
                        gl=True ) ## gl for global
    
    def hideSeams(self):
        print( 'Hiding seams.'  )
        pm.polyOptions( displayMapBorder=False, gl=True ) ## gl for global
        
    def runUvXformsTool(self):
        self.uvXformsTool = TexturerUvXformsTool.TexturerUvXformsTool(self, auto_ui=True)
        
    def roadkill(self):
        self.runRoadkill()
        
    def runRoadkill(self):
        print( 'Running Roadkill via mel:  "MmmmToolsRoadKill;" ' )
        try:
            pm.mel.eval('MmmmToolsRoadKill;')
        except:
            traceback.print_exc()